"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const RandomSeed = require("random-seed");
const defaultSpec = {
    seed: "1",
    nodeCountRange: {
        min: 50,
        max: 100
    },
    distribution: {
        float: 5,
        int: 5,
        null: 2,
        string: 10,
        object: 4,
        array: 1,
        branchDone: 5
    },
    propPrefixes: ["prop", "longerPropName", "p"],
    stringValues: ["John Smith", "blue", "This is a sentence.", "Random string"],
    floatRange: {
        min: -1000000000,
        max: 1000000000
    },
    intRange: {
        min: -1000000000,
        max: 1000000000
    }
};
function selectRandomKey(distribution, random) {
    const sum = _.sum(_.values(distribution));
    const target = random.floatBetween(0, sum);
    let result;
    let currentSum = 0;
    _.each(distribution, (value, key) => {
        currentSum += value;
        if (target < currentSum) {
            result = key;
            return false;
        }
    });
    return result;
}
exports.selectRandomKey = selectRandomKey;
function selectRandomValue(values, random) {
    const index = random.range(values.length);
    return values[index];
}
function makeValue(spec, distribution, random) {
    const valueType = selectRandomKey(distribution, random);
    switch (valueType) {
        case "float":
            return random.floatBetween(spec.floatRange.min, spec.floatRange.max);
        case "int":
            return random.intBetween(spec.floatRange.min, spec.floatRange.max);
        case "null":
            return null;
        case "string":
            return selectRandomValue(spec.stringValues, random);
        case "object":
            return {};
        case "array":
            return [];
        case "branchDone":
        default:
            return undefined;
    }
}
function makeNode(obj, spec, currentNodeCount, targetNodeCount, random) {
    while (currentNodeCount < targetNodeCount) {
        const value = makeValue(spec, spec.distribution, random);
        if (value === undefined) { // branchDone
            return currentNodeCount;
        }
        currentNodeCount += 1;
        if (_.isObject(value)) {
            currentNodeCount = makeNode(value, spec, currentNodeCount, targetNodeCount, random);
        }
        if (_.isArray(obj)) {
            obj.push(value);
        }
        else {
            const propName = selectRandomValue(spec.propPrefixes, random) + currentNodeCount;
            obj[propName] = value;
        }
    }
    return currentNodeCount;
}
function create(partialSpec, rand) {
    const spec = _.cloneDeep(defaultSpec);
    _.assign(spec, partialSpec);
    const random = rand ? rand : RandomSeed.create(spec.seed);
    const targetNodeCount = random.intBetween(spec.nodeCountRange.min, spec.nodeCountRange.max);
    if (targetNodeCount === 1) {
        const distribution = _.omit(spec.distribution, "branchDone");
        return makeValue(spec, distribution, random);
    }
    const distribution = _.pick(spec.distribution, "array", "object");
    let result = makeValue(spec, distribution, random);
    let currentNodeCount = 1;
    while (currentNodeCount < targetNodeCount) {
        currentNodeCount = makeNode(result, spec, currentNodeCount, targetNodeCount, random);
    }
    return result;
}
exports.create = create;
//# sourceMappingURL=RandomObject.js.map